package Common;

use strict;
use warnings;
use DBI;

my $Class_Data = 'data';
my $Class_Result = 'result';

my $Db_String = 'dbi:SQLite:';

sub execute {
    my @lines = @_;

    while (my $line = shift @lines) {
        # [table]o܂œǂݔ΂
        last if $line =~ /^\[table\]/;
    }

    my $test_tables = [];
    my $table = '';
    while (my $line = shift @lines) {
        if ($line =~ /^$/) {
            push @$test_tables, $table;
            $table = '';
            next;
        }
        last if $line =~ /^\[sql\]/;
        $table .= $line;
    }

    my $sql_execute = '';
    while (my $line = shift @lines) {
        next if $line =~ /^$/;
        $sql_execute .= $line;
    }
    
    execute_sqldata($test_tables, $sql_execute);
}

sub execute_sqldata {
    my $test_tables = shift;
    my $sql_execute = shift;

    print_header();

    my $dbh = get_connect();
    foreach my $test_data (@$test_tables) {
        my @data = split /\n/, $test_data;
        my ($table, $headers, $items) = get_data(@data);
        my $columns = get_columns($headers);
    
        my $sql_create = get_sql_for_create($table, $headers);
        my $sql_insert = get_sql_for_insert($table, $columns);
    
        table_create($dbh, $sql_create);

        show_data($table, $columns, $items, $Class_Data);
        insert_data($dbh, $items, $sql_insert);
    }
    show_sql($sql_execute);
    print "\n";
    my ($result_columns, $result) = select_data($dbh, $sql_execute);
    show_data('s', $result_columns, $result, $Class_Result);

    print_footer();
    
    $dbh->disconnect;
}

sub get_columns {
    my $headers = shift;

    return [map {(split /\s/, $_)[0]} @$headers];
}

sub get_sql_for_create {
    my $table = shift;
    my $columns = shift;

    return sprintf "create table %s (%s)", $table, join(', ', @$columns);
}

sub get_sql_for_insert {
    my $table = shift;
    my $columns = shift;

    my $columns_num = @$columns;
    return sprintf "insert into %s (%s) values (%s)",
        $table,
        join(', ', @$columns),
        join(', ', ('?') x $columns_num);
}

sub get_connect {
    my $dbh = DBI->connect($Db_String, undef, undef, {
        AutoCommit => 0,
        RaiseError => 1,
    });
    return $dbh;
}

sub table_create {
    my $dbh = shift;
    my $sql = shift;

    $dbh->do($sql);
    #$dbh->commit;
}

sub get_data {
    my @data = @_;

    my $table = shift @data;
    my $top = shift @data;
    my @headers = split /,/, $top;

    my @items = ();
    foreach my $line (@data) {
        push @items , [split /,/, $line];
    }
    return ($table, \@headers, \@items);
}

sub show_data {
    my $caption = shift;
    my $headers = shift;
    my $items = shift;
    my $class = shift;

    if ($caption ne '') {
        printf qq{<div class="caption">%s</div>\n}, html_escape($caption);
    }
    print qq{<table class="$class">\n};
    print "<tr>\n";
    foreach my $item (@$headers) {
        printf "  <th>%s</th>\n", html_escape($item);
    }
    print "</tr>\n";
    foreach my $row (@$items) {
        print "<tr>\n";
        foreach my $item (@$row) {
            printf "  <td>%s</td>\n", html_escape($item);
        }
        print "</tr>\n";
    }
    print "</table>\n";
    print "\n";
}

sub show_sql {
    my $sql = shift;
    printf qq{<div class="sqlcaption">SQL</div>};
    print qq{<div class="sql">};
    print html_escape($sql);
    print qq{</div>\n};
}

sub insert_data {
    my $dbh = shift;
    my $items = shift;
    my $sql = shift;

    my $sth = $dbh->prepare($sql);
    foreach my $row (@$items) {
        $sth->execute(@$row);
    }
    #$dbh->commit;
}

sub select_data {
    my $dbh = shift;
    my $sql = shift;

    my $sth = $dbh->prepare($sql);
    $sth->execute();
    my $columns = $sth->{NAME};
    my $result = $sth->fetchall_arrayref;
    return ($columns, $result);
}

sub html_escape {
    my $str = shift;

    $str =~ s/&/&amp;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    $str =~ s/"/&quot;/g;

    return $str;
}

sub print_header {
    print <<EOD;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html lang="ja">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=Shift_JIS">
<style type="text/css">
<!--
div.caption, div.sqlcaption {
  color: midnightblue;
  font-size: small;
  font-weight: bold;
  margin: 10px 0 0 10px;
  padding: 0.25em;
}
div.sql {
  font-size: small;
  margin: 0 10px 1px 10px;
  padding: 0.25em;
  background-color: palegreen;
  border-collapse: collapse;
  border-style: solid;
  border-width: 2px;
  border-color: darkgreen;
  white-space: pre;
}
table {
  margin: 0 10px 1px 10px;
  font-size: small;
  border-collapse: collapse;
  border-style: solid;
  border-width: 2px;
  border-color: darkgreen;
}
td {
  border-color: darkgreen;
  border-style: solid;
  border-width: 1px;
  padding: 3pt;
  vertical-align: top;
}
th {
  background-color: lightgreen;
  border-color: darkgreen;
  border-style: solid;
  border-width: 1px;
  font-weight: normal;
  padding: 3pt;
  vertical-align: top;
}
-->
</style>
<title>SQLs</title>
</head>
<body>
EOD
}

sub print_footer {
    print <<EOD;
</body>
</html>
EOD
}
 
1;
